/****************************************************************************
** Author: Nikola Markovic
**
** Copyright (C) 2022 MikroElektronika d.o.o.
** Contact: https://www.mikroe.com/contact
**
** This file is part of the mikroSDK package
**
** Commercial License Usage
**
** Licensees holding valid commercial NECTO compilers AI licenses may use this
** file in accordance with the commercial license agreement provided with the
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and The mikroElektronika Company.
** For licensing terms and conditions see
** https://www.mikroe.com/legal/software-license-agreement.
** For further information use the contact form at
** https://www.mikroe.com/contact.
**
**
** GNU Lesser General Public License Usage
**
** Alternatively, this file may be used for
** non-commercial projects under the terms of the GNU Lesser
** General Public License version 3 as published by the Free Software
** Foundation: https://www.gnu.org/licenses/lgpl-3.0.html.
**
** The above copyright notice and this permission notice shall be
** included in all copies or substantial portions of the Software.
**
** THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
** OF MERCHANTABILITY, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
** TO THE WARRANTIES FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
** IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
** DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT
** OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
** OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
**
****************************************************************************/
/* Project name:
 *   Analog input
 * Copyright:
 *   (c) Mikroelektronika, 2022.
 * Description:
 *   Example is meant for testing ADC functionality using mikroSDK 2.0. The
 *   read value from analog pin on MIKROBUS 1 is shown on PORT A.
 */

// ------------------------------------------------------------------ INCLUDES
#include "drv_analog_in.h"
#include "board.h"
#include "drv_port.h"

// If hardware without mikroBUS socket is used, make sure to define adequate pin.
#ifndef MIKROBUS_1_AN
#define MIKROBUS_1_AN NC
#endif

// Voltage reference value. It can be changed in CODEGRIP: POWER -> Outputs -> Voltage Reference -> Set Voltage.
#define ADC_VREF_VALUE  4

// Error value.
#define ADC_ERROR_VALUE 0xAA

#ifndef PORT_A
#define PORT_A NC
#endif

// ------------------------------------------------------------------ VARIABLES
static analog_in_t analog_in;               // Analog input driver context structure.
static analog_in_config_t analog_in_cfg;    // Analog input configuration structure.
static port_t configured_port;

int main() {
    uint16_t read_data_buf;                 // Analog input data buffer.

    port_init( &configured_port, PORT_A, ( uint8_t )0xFF, (pin_direction_t)GPIO_DIGITAL_OUTPUT );

    analog_in_configure_default( &analog_in_cfg );
    analog_in_cfg.input_pin  = MIKROBUS_1_AN;
    analog_in_cfg.resolution = ANALOG_IN_RESOLUTION_12_BIT;
    analog_in_cfg.vref_value = ADC_VREF_VALUE;

    if( ADC_ERROR == analog_in_open( &analog_in, &analog_in_cfg ) ) {
        // Error handling.
        return -1;
    }

    while( 1 ) {
        if( ADC_ERROR == analog_in_read( &analog_in, &read_data_buf ) ) {
            port_write( &configured_port, ADC_ERROR_VALUE );
        }

        port_write( &configured_port, read_data_buf & 0xFF );
    }

    return 0;
}
